import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, dict_output


@click.command('run_jobs')
@click.option(
    "--wait",
    help="Wait for job completion",
    default="True",
    show_default=True,
    is_flag=True
)
@pass_context
@custom_exception
@dict_output
def cli(ctx, wait=True):
    """Run jobs in queue. There is no way to trigger a single job execution.

Output:

    Job information
    """
    return ctx.gi.job.run_jobs(wait=wait)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, list_output


@click.command('get_organisms')
@click.option(
    "--organism_id",
    help="An organism ID",
    type=str
)
@click.option(
    "--genus",
    help="The genus of the organism",
    type=str
)
@click.option(
    "--species",
    help="The species of the organism",
    type=str
)
@click.option(
    "--common",
    help="The common name of the organism",
    type=str
)
@click.option(
    "--abbr",
    help="The abbreviation of the organism",
    type=str
)
@click.option(
    "--comment",
    help="A comment / description",
    type=str
)
@pass_context
@custom_exception
@list_output
def cli(ctx, organism_id="", genus="", species="", common="", abbr="", comment=""):
    """Get organisms

Output:

    Organism information
    """
    return ctx.gi.organism.get_organisms(organism_id=organism_id, genus=genus, species=species, common=common, abbr=abbr, comment=comment)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, list_output


@click.command('get_taxonomic_ranks')
@pass_context
@custom_exception
@list_output
def cli(ctx):
    """Get taxonomic ranks

Output:

    Taxonomic ranks
    """
    return ctx.gi.organism.get_taxonomic_ranks()



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, str_output


@click.command('load_go')
@click.argument("name", type=str)
@click.argument("program", type=str)
@click.argument("programversion", type=str)
@click.argument("sourcename", type=str)
@click.argument("gaf_output", type=str)
@click.option(
    "--gaf_ext",
    help="If looking for files in a directory, extension of the GAF files",
    type=str
)
@click.option(
    "--query_type",
    help="The feature type (e.g. 'gene', 'mRNA', 'contig') of the query. It must be a valid Sequence Ontology term.",
    type=str
)
@click.option(
    "--query_uniquename",
    help="Use this if the --query-re regular expression matches unique names instead of names in the database.",
    is_flag=True
)
@click.option(
    "--method",
    help="Import method ('add' or 'remove')",
    default="add",
    show_default=True,
    type=str
)
@click.option(
    "--re_name",
    help="Regular expression to extract the feature name from GAF file.",
    type=str
)
@click.option(
    "--no_wait",
    help="Do not wait for job to complete",
    is_flag=True
)
@click.option(
    "--algorithm",
    help="analysis algorithm",
    type=str
)
@click.option(
    "--sourceversion",
    help="analysis sourceversion",
    type=str
)
@click.option(
    "--sourceuri",
    help="analysis sourceuri",
    type=str
)
@click.option(
    "--description",
    help="analysis description",
    type=str
)
@click.option(
    "--date_executed",
    help="analysis date_executed (yyyy-mm-dd)",
    type=str
)
@pass_context
@custom_exception
@str_output
def cli(ctx, name, program, programversion, sourcename, gaf_output, gaf_ext="", query_type="", query_uniquename=False, method="add", re_name="", no_wait=False, algorithm="", sourceversion="", sourceuri="", description="", date_executed=""):
    """Create a GO analysis

Output:

    Loading information
    """
    return ctx.gi.analysis.load_go(name, program, programversion, sourcename, gaf_output, gaf_ext=gaf_ext, query_type=query_type, query_uniquename=query_uniquename, method=method, re_name=re_name, no_wait=no_wait, algorithm=algorithm, sourceversion=sourceversion, sourceuri=sourceuri, description=description, date_executed=date_executed)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, str_output


@click.command('populate_mviews')
@click.option(
    "--name",
    help="filter on mview name",
    type=str
)
@click.option(
    "--no_wait",
    help="Do not wait for job to complete",
    is_flag=True
)
@pass_context
@custom_exception
@str_output
def cli(ctx, name="", no_wait=""):
    """Populate materialized views

Output:

    Loading information
    """
    return ctx.gi.db.populate_mviews(name=name, no_wait=no_wait)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from collections import OrderedDict
import logging
from tripal.client import Client

logging.getLogger("requests").setLevel(logging.CRITICAL)
log = logging.getLogger()


class OrganismClient(Client):
    CLIENT_BASE = '/tripal_api/'

    def get_organism_nodes(self, node=None):
        """
        Get organism nodes

        :type node: int
        :param node: filter on node id

        :rtype: list of dict
        :return: Organism node information
        """

        if node:
            nodes = [self._get('node/%s' % node, {})]
        else:
            nodes = self._get('node', {})

        nodes = [n for n in nodes if n['type'] == 'chado_organism']

        return nodes

    def get_organisms(self, organism_id=None, genus=None, species=None, common=None, abbr=None,
                      comment=None):
        """
        Get organisms

        :type organism_id: str
        :param organism_id: An organism ID

        :type genus: str
        :param genus: The genus of the organism

        :type common: str
        :param common: The common name of the organism

        :type abbr: str
        :param abbr: The abbreviation of the organism

        :type species: str
        :param species: The species of the organism

        :type comment: str
        :param comment: A comment / description

        :rtype: list of dict
        :return: Organism information
        """

        orgs = self._request('chado/list', {'table': 'organism'})
        if organism_id:
            orgs = [v for v in orgs if v['organism_id'] == organism_id]
        if genus:
            orgs = [v for v in orgs if v['genus'] == genus]
        if species:
            orgs = [v for v in orgs if v['species'] == species]
        if common:
            orgs = [v for v in orgs if v['common_name'] == common]
        if abbr:
            orgs = [v for v in orgs if v['abbreviation'] == abbr]
        if comment:
            orgs = [v for v in orgs if v['comment'] == comment]

        return orgs

    def add_organism(self, genus, common, abbr, species=None, comment=None,
                     infraspecific_rank=None, infraspecific_name=None):
        """
        Add a new organism to the database

        :type genus: str
        :param genus: The genus of the organism

        :type common: str
        :param common: The common name of the organism

        :type abbr: str
        :param abbr: The abbreviation of the organism

        :type species: str
        :param species: The species of the organism

        :type comment: str
        :param comment: A comment / description

        :type infraspecific_rank: str
        :param infraspecific_rank: The type name of infraspecific name for any taxon below the rank of species. Must be one of ['subspecies', 'varietas', 'subvariety', 'forma', 'subforma']

        :type infraspecific_name: str
        :param infraspecific_name: The infraspecific name for this organism.

        :rtype: dict
        :return: Organism information
        """

        if (infraspecific_rank or infraspecific_name) and not (infraspecific_name and infraspecific_rank):
            raise Exception("You should specific both infraspecific_rank and infraspecific_name, or none of them.")

        if infraspecific_rank and infraspecific_rank not in ['subspecies', 'varietas', 'subvariety', 'forma', 'subforma']:
            raise Exception("infraspecific_rank must be one of ['subspecies', 'varietas', 'subvariety', 'forma', 'subforma']")

        params = {
            'type': 'chado_organism',
            'genus': genus,
            'species': species,
            'abbreviation': abbr,
            'common_name': common,
            'description': comment,
            'type_id': 0,
            'infraspecific_name': '',
        }

        if infraspecific_rank:
            params['infraspecific_name'] = infraspecific_name
            allowed_ranks = self.get_taxonomic_ranks()

            for r in allowed_ranks:
                if r['name'] == infraspecific_rank:
                    params['type_id'] = int(r['cvterm_id'])
                    break

        return self._request('node', params)

    def get_taxonomic_ranks(self):
        """
        Get taxonomic ranks

        :rtype: list of dict
        :return: Taxonomic ranks
        """

        return self._request('chado/taxonomic_ranks', {})

    def sync(self, organism=None, organism_id=None, job_name=None, no_wait=None):
        """
        Synchronize an organism

        :type organism: str
        :param organism: Common name of the organism to sync

        :type organism_id: str
        :param organism_id: ID of the organism to sync

        :type job_name: str
        :param job_name: Name of the job

        :type no_wait: bool
        :param no_wait: Return immediately without waiting for job completion

        :rtype: str
        :return: status
        """

        if organism_id:
            found_org = self.get_organisms(organism_id=organism_id)
            if not found_org:
                raise Exception("Invalid organism ID")
        elif organism:
            found_org = self.get_organisms(common=organism)
            if not found_org:
                found_org = self.get_organisms(abbr=organism)

            if not found_org:
                raise Exception("Invalid organism name")

            organism_id = found_org[0]['organism_id']

        if not organism_id:
            raise Exception("Either organism or organism_id is required")

        if not job_name:
            job_name = 'Sync Organism'

        job_args = OrderedDict()
        job_args['base_table'] = 'organism'
        job_args['max_sync'] = ''
        job_args['organism_id'] = ''
        job_args['types'] = []
        job_args['ids'] = [organism_id]
        job_args['linking_table'] = 'chado_organism'
        job_args['node_type'] = 'chado_organism'

        r = self.tripal.job.add_job(job_name, 'chado_feature', 'chado_node_sync_records', job_args)
        if 'job_id' not in r or not r['job_id']:
            raise Exception("Failed to create job, received %s" % r)

        if no_wait:
            return r
        else:
            return self._run_job_and_wait(r['job_id'])



import os

import click

from tripal import TripalInstance
from tripaille.cli import pass_context
from tripaille import config
from tripaille.io import warn, info

CONFIG_TEMPLATE = """## Tripal's tripaille: Global Configuration File.
# Each stanza should contain a single Tripal server to control.
#
# You can set the key __default to the name of a default instance
__default: local

local:
    tripal_url: "%(tripal_url)s"
    username: "%(username)s"
    password: "%(password)s"
    auth_login: "%(auth_login)s" # Optional, only if your tripal instance is running behind an authentication proxy
    auth_password: "%(auth_password)s" # Optional, only if your tripal instance is running behind an authentication proxy
"""

SUCCESS_MESSAGE = (
    "Ready to go! Type `tripaille` to get a list of commands you can execute."
)


@click.command("config_init")
@pass_context
def cli(ctx, url=None, api_key=None, admin=False, **kwds):
    """Help initialize global configuration (in home directory)
    """
    # TODO: prompt for values someday.
    click.echo("""Welcome to Tripal's Tripaille!""")
    if os.path.exists(config.global_config_path()):
        info("Your tripaille configuration already exists. Please edit it instead: %s" % config.global_config_path())
        return 0

    while True:
        # Check environment
        tripal_url = click.prompt("TRIPAL_URL")
        username = click.prompt("TRIPAL_USER")
        password = click.prompt("TRIPAL_PASS", hide_input=True)
        auth_login = ''
        auth_password = ''
        if click.confirm("""Is your tripal instance running behind an authentication proxy?"""):
            auth_login = click.prompt("TRIPAL_AUTH_USER")
            auth_password = click.prompt("TRIPAL_AUTH_PASS", hide_input=True)

        info("Testing config...")
        try:
            TripalInstance(tripal_url=tripal_url, username=username, password=password)
            # We do a connection test during startup.
            info("Ok! Everything looks good.")
            break
        except Exception as e:
            warn("Error, we could not access the configuration data for your instance: %s", e)
            should_break = click.prompt("Continue despite inability to contact this instance? [y/n]")
            if should_break in ('Y', 'y'):
                break

    config_path = config.global_config_path()
    if os.path.exists(config_path):
        warn("File %s already exists, refusing to overwrite." % config_path)
        return -1

    with open(config_path, "w") as f:
        f.write(CONFIG_TEMPLATE % {
            'tripal_url': tripal_url,
            'username': username,
            'password': password,
            'auth_login': auth_login,
            'auth_password': auth_password,
        })
        info(SUCCESS_MESSAGE)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, str_output


@click.command('index')
@click.option(
    "--mode",
    help="Indexing mode: 'website' to index everything, 'table' to index a single table (default: website)",
    default="website",
    show_default=True,
    type=str
)
@click.option(
    "--table",
    help="Table to index (only in 'table' mode)",
    type=str
)
@click.option(
    "--index_name",
    help="Index name (only in 'table' mode)",
    type=str
)
@click.option(
    "--queues",
    help="Number of indexing task queues",
    default="10",
    show_default=True,
    type=int
)
@click.option(
    "--fields",
    help="Fields to index (only in 'table' mode), syntax: <field_name>|<field_type>, field_type should be one of 'string', 'keyword', 'date', 'long', 'double', 'boolean', 'ip', 'object', 'nested', 'geo_point', 'geo_shape', or 'completion'",
    type=str,
    multiple=True
)
@click.option(
    "--links",
    help="List of links to show to users, syntax: <column-where-to-show-the-link>|</your/url/[any-column-name]>",
    type=str,
    multiple=True
)
@click.option(
    "--tokenizer",
    help="Tokenizer to use (one of 'standard', 'letter', 'lowercase', 'whitespace', 'uax_url_email', 'classic', 'ngram', 'edge_ngram', 'keyword', 'pattern', or 'path_hierarchy'; default='standard')",
    default="standard",
    show_default=True,
    type=str
)
@click.option(
    "--job_name",
    help="Name of the job",
    type=str
)
@click.option(
    "--no_wait",
    help="Do not wait for job to complete",
    is_flag=True
)
@pass_context
@custom_exception
@str_output
def cli(ctx, mode="website", table="", index_name="", queues=10, fields=None, links={}, tokenizer="standard", job_name="", no_wait=False):
    """Schedule database indexing using elasticsearch

Output:

    Indexing information
    """
    return ctx.gi.db.index(mode=mode, table=table, index_name=index_name, queues=queues, fields=fields, links=links, tokenizer=tokenizer, job_name=job_name, no_wait=no_wait)



import click
from tripaille.commands.job.add_job import cli as func0
from tripaille.commands.job.get_jobs import cli as func1
from tripaille.commands.job.get_logs import cli as func2
from tripaille.commands.job.run_jobs import cli as func3
from tripaille.commands.job.wait import cli as func4


@click.group()
def cli():
    pass


cli.add_command(func0)
cli.add_command(func1)
cli.add_command(func2)
cli.add_command(func3)
cli.add_command(func4)



import click
from tripaille.commands.organism.add_organism import cli as func0
from tripaille.commands.organism.get_organism_nodes import cli as func1
from tripaille.commands.organism.get_organisms import cli as func2
from tripaille.commands.organism.get_taxonomic_ranks import cli as func3
from tripaille.commands.organism.sync import cli as func4


@click.group()
def cli():
    pass


cli.add_command(func0)
cli.add_command(func1)
cli.add_command(func2)
cli.add_command(func3)
cli.add_command(func4)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, str_output


@click.command('load_gff3')
@click.argument("gff", type=str)
@click.option(
    "--organism",
    help="Organism common name or abbreviation",
    type=str
)
@click.option(
    "--organism_id",
    help="Organism ID",
    type=int
)
@click.option(
    "--analysis",
    help="Analysis name",
    type=str
)
@click.option(
    "--analysis_id",
    help="Analysis ID",
    type=int
)
@click.option(
    "--import_mode",
    help="Import mode (add_only=existing features won't be touched, update=existing features will be updated and obsolete attributes kept, refresh=existing features will be updated and obsolete attributes removed, remove=features present in the db and in the GFF3 file will be removed)')",
    default="update",
    show_default=True,
    type=str
)
@click.option(
    "--target_organism",
    help="In case of Target attribute in the GFF3, choose the organism abbreviation or common name to which target sequences belong. Select this only if target sequences belong to a different organism than the one specified with --organism-id. And only choose an organism here if all of the target sequences belong to the same species. If the targets in the GFF file belong to multiple different species then the organism must be specified using the 'target_organism=genus:species' attribute in the GFF file.')",
    type=str
)
@click.option(
    "--target_organism_id",
    help="In case of Target attribute in the GFF3, choose the organism ID to which target sequences belong. Select this only if target sequences belong to a different organism than the one specified with --organism-id. And only choose an organism here if all of the target sequences belong to the same species. If the targets in the GFF file belong to multiple different species then the organism must be specified using the 'target_organism=genus:species' attribute in the GFF file.')",
    type=int
)
@click.option(
    "--target_type",
    help="In case of Target attribute in the GFF3, if the unique name for a target sequence is not unique (e.g. a protein and an mRNA have the same name) then you must specify the type for all targets in the GFF file. If the targets are of different types then the type must be specified using the 'target_type=type' attribute in the GFF file. This must be a valid Sequence Ontology (SO) term.')",
    type=str
)
@click.option(
    "--target_create",
    help="In case of Target attribute in the GFF3, if the target feature cannot be found, create one using the organism and type specified above, or using the 'target_organism' and 'target_type' fields specified in the GFF file. Values specified in the GFF file take precedence over those specified above.')",
    is_flag=True
)
@click.option(
    "--start_line",
    help="The line in the GFF file where importing should start",
    type=int
)
@click.option(
    "--landmark_type",
    help="A Sequence Ontology type for the landmark sequences in the GFF fie (e.g. 'chromosome').",
    type=str
)
@click.option(
    "--alt_id_attr",
    help="When ID attribute is absent, specify which other attribute can uniquely identify the feature.",
    type=str
)
@click.option(
    "--create_organism",
    help="Create organisms when encountering organism attribute (these lines will be skip otherwise)",
    is_flag=True
)
@click.option(
    "--re_mrna",
    help="Regular expression for the mRNA name",
    type=str
)
@click.option(
    "--re_protein",
    help="Replacement string for the protein name",
    type=str
)
@click.option(
    "--job_name",
    help="Name of the job",
    type=str
)
@click.option(
    "--no_wait",
    help="Do not wait for job to complete",
    is_flag=True
)
@pass_context
@custom_exception
@str_output
def cli(ctx, gff, organism="", organism_id="", analysis="", analysis_id="", import_mode="update", target_organism="", target_organism_id="", target_type="", target_create=False, start_line="", landmark_type="", alt_id_attr="", create_organism="", re_mrna="", re_protein="", job_name="", no_wait=False):
    """Load GFF3 file

Output:

    Loading information
    """
    return ctx.gi.analysis.load_gff3(gff, organism=organism, organism_id=organism_id, analysis=analysis, analysis_id=analysis_id, import_mode=import_mode, target_organism=target_organism, target_organism_id=target_organism_id, target_type=target_type, target_create=target_create, start_line=start_line, landmark_type=landmark_type, alt_id_attr=alt_id_attr, create_organism=create_organism, re_mrna=re_mrna, re_protein=re_protein, job_name=job_name, no_wait=no_wait)



import click
from tripaille.cli import pass_context
from tripaille.decorators import custom_exception, str_output


@click.command('load_fasta')
@click.argument("fasta", type=str)
@click.option(
    "--organism",
    help="Organism common name or abbreviation",
    type=str
)
@click.option(
    "--organism_id",
    help="Organism ID",
    type=int
)
@click.option(
    "--analysis",
    help="Analysis name",
    type=str
)
@click.option(
    "--analysis_id",
    help="Analysis ID",
    type=int
)
@click.option(
    "--sequence_type",
    help="Sequence type",
    default="contig",
    show_default=True,
    type=str
)
@click.option(
    "--re_name",
    help="Regular expression for the name",
    type=str
)
@click.option(
    "--re_uniquename",
    help="Regular expression for the unique name",
    type=str
)
@click.option(
    "--db_ext_id",
    help="External DB ID",
    type=str
)
@click.option(
    "--re_accession",
    help="Regular expression for the accession from external DB",
    type=str
)
@click.option(
    "--rel_type",
    help="Relation type (part_of or derives_from)",
    type=str
)
@click.option(
    "--rel_subject_re",
    help="Relation subject regular expression (used to extract id of related entity)",
    type=str
)
@click.option(
    "--rel_subject_type",
    help="Relation subject type (must match already loaded data, e.g. mRNA)",
    type=str
)
@click.option(
    "--method",
    help="Insertion method (insert, update or insup, default=insup (Insert and Update))",
    default="insup",
    show_default=True,
    type=str
)
@click.option(
    "--match_type",
    help="Match type for already loaded features (name or uniquename; default=uniquename; used for \"Update only\" or \"Insert and update\" methods)'",
    default="uniquename",
    show_default=True,
    type=str
)
@click.option(
    "--job_name",
    help="Name of the job",
    type=str
)
@click.option(
    "--no_wait",
    help="Do not wait for job to complete",
    is_flag=True
)
@pass_context
@custom_exception
@str_output
def cli(ctx, fasta, organism="", organism_id="", analysis="", analysis_id="", sequence_type="contig", re_name="", re_uniquename="", db_ext_id="", re_accession="", rel_type="", rel_subject_re="", rel_subject_type="", method="insup", match_type="uniquename", job_name="", no_wait=False):
    """Load fasta sequences

Output:

    Loading information
    """
    return ctx.gi.analysis.load_fasta(fasta, organism=organism, organism_id=organism_id, analysis=analysis, analysis_id=analysis_id, sequence_type=sequence_type, re_name=re_name, re_uniquename=re_uniquename, db_ext_id=db_ext_id, re_accession=re_accession, rel_type=rel_type, rel_subject_re=rel_subject_re, rel_subject_type=rel_subject_type, method=method, match_type=match_type, job_name=job_name, no_wait=no_wait)



from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
import logging
from tripal.analysis import AnalysisClient
from tripal.db import DbClient
from tripal.feature import FeatureClient
from tripal.job import JobClient
from tripal.organism import OrganismClient

from future import standard_library
standard_library.install_aliases()

logging.getLogger("requests").setLevel(logging.CRITICAL)
log = logging.getLogger()


class TripalInstance(object):

    def __init__(self, tripal_url="http://localhost/tripal/", username="admin", password="changeme", auth_login=None, auth_password=None, **kwargs):

        self.tripal_url = tripal_url
        self.username = username
        self.password = password

        self.auth_login = auth_login
        self.auth_password = auth_password

        self.analysis = AnalysisClient(self)
        self.db = DbClient(self)
        self.feature = FeatureClient(self)
        self.job = JobClient(self)
        self.organism = OrganismClient(self)

    def __str__(self):
        return '<TripalInstance at %s>' % self.tripal_url
